class CommandError(Exception):
    pass


class VaultError(Exception):
    pass


class VerificationError(Exception):
    pass


class ZymkeyLibraryError(Exception):
    pass


class ZymkeyTimeoutError(Exception):
    pass

